﻿#ifndef SIMULATIONCONTROL_H
#define SIMULATIONCONTROL_H

#include <QObject>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonValue>
#include "platform.h"
#include "external/radar.h"
#include "external/jamming.h"
#include "QtWebSockets/QWebSocket"

class SimulationControl : public QObject
{
    Q_OBJECT
public:
    std::vector<Platform *> allyplane;
    std::vector<Platform *> allymissile;
    std::map<int, Jamming *> allyJammingSim;
    std::vector<std::vector<Platform *>> ally;

    std::vector<Platform *> enemyradar;
    std::map<int, Radar *> enemyThaadRadarSim;
    std::vector<Platform *> enemysat;
    std::vector<std::vector<Platform *>> enemy;

    explicit SimulationControl(QObject *parent = nullptr);

signals:
    void SigReturnData(QWebSocket *pSender, QJsonObject *obj);

public slots:
    void SlotGetInit(QWebSocket *pSender, QJsonObject obj);
    void SlotGetLeadPost(QWebSocket *pSender, QJsonObject obj);
    void SlotGetTaskPlan(QWebSocket *pSender, QJsonObject obj);
    void SlotGetGlobalMap(QWebSocket *pSender, QJsonObject obj);
    void SlotGetPlatformTable(QWebSocket *pSender, QJsonObject obj);
    void SlotGetInstanceTable(QWebSocket *pSender, QJsonObject obj);
    void SlotGetRadarInstances(QWebSocket *pSender, QJsonObject obj);
    void SlotGetRadarParams(QWebSocket *pSender, QJsonObject obj);
    void SlotGetThaadInstances(QWebSocket *pSender, QJsonObject obj);
    void SlotGetThaadParams(QWebSocket *pSender, QJsonObject obj);
    void SlotSetThaadStyle(QWebSocket *pSender, QJsonObject obj);
    void SlotGetJammingInstances(QWebSocket *pSender, QJsonObject obj);
    void SlotGetJammingParams(QWebSocket *pSender, QJsonObject obj);
    void SlotSetJammingStyle(QWebSocket *pSender, QJsonObject obj);
    void SlotGetESMParams(QWebSocket *pSender, QJsonObject obj);
};

#endif // SIMULATIONCONTROL_H
